<div class="cf-turnstile"
     data-sitekey="{{ widget.site_key }}"
     data-theme="{{ widget.theme }}"
     data-size="{{ widget.size }}"
     {% if widget.appearance %}data-appearance="{{ widget.appearance }}"{% endif %}
     {% if widget.callback %}data-callback="{{ widget.callback }}"{% endif %}>
</div>
